import { DOCUMENT } from '@angular/common';
import * as i0 from '@angular/core';
import { Injectable, Directive, Inject, Optional, HostListener, NgModule } from '@angular/core';
import { ReplaySubject, mergeMap, EMPTY } from 'rxjs';
import * as i2 from '@angular/router';

class PreloadOnDemandService {
    constructor() {
        this.preloadOnDemandSource$ = new ReplaySubject(1);
        this.preloadOnDemand$ = this.preloadOnDemandSource$.asObservable();
    }
    startPreload(routePath) {
        this.preloadOnDemandSource$.next(routePath);
    }
}
PreloadOnDemandService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: PreloadOnDemandService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
PreloadOnDemandService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: PreloadOnDemandService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: PreloadOnDemandService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }] });

/* eslint-disable @angular-eslint/directive-selector */
class HoverPreloadLinkDirective {
    constructor(onDemandPreloadService, document, link, linkWithHref) {
        this.onDemandPreloadService = onDemandPreloadService;
        this.document = document;
        this.window = this.document.defaultView;
        this.routerLink = link || linkWithHref;
    }
    onMouseEnter() {
        // Check whether we have the window. If so, use the requestIdleCallback and call our prefetch function.
        if (this.window) {
            this.window.requestIdleCallback(() => this.prefetch());
        }
        else {
            // If we don't have the window variable, just call the prefetch function directly.
            this.prefetch();
        }
    }
    prefetch() {
        const path = this.getPath(this.routerLink);
        if (!!path) {
            this.onDemandPreloadService.startPreload(path);
        }
    }
    getPath(routerLink) {
        if ("href" in routerLink && routerLink.href) {
            return routerLink.href;
        }
        else if ("urlTree" in routerLink && routerLink.urlTree) {
            return routerLink.urlTree.toString();
        }
        else {
            return null;
        }
    }
}
HoverPreloadLinkDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: HoverPreloadLinkDirective, deps: [{ token: PreloadOnDemandService }, { token: DOCUMENT }, { token: i2.RouterLink, optional: true }, { token: i2.RouterLinkWithHref, optional: true }], target: i0.ɵɵFactoryTarget.Directive });
HoverPreloadLinkDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "14.2.0", type: HoverPreloadLinkDirective, selector: "[routerLink]", host: { listeners: { "mouseenter": "onMouseEnter()" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: HoverPreloadLinkDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[routerLink]',
                }]
        }], ctorParameters: function () {
        return [{ type: PreloadOnDemandService }, { type: Document, decorators: [{
                        type: Inject,
                        args: [DOCUMENT]
                    }] }, { type: i2.RouterLink, decorators: [{
                        type: Optional
                    }] }, { type: i2.RouterLinkWithHref, decorators: [{
                        type: Optional
                    }] }];
    }, propDecorators: { onMouseEnter: [{
                type: HostListener,
                args: ['mouseenter']
            }] } });

class HoverPreloadStrategy {
    constructor(preloadOnDemandService) {
        this.preloadOnDemandService = preloadOnDemandService;
    }
    preload(route, load) {
        return this.preloadOnDemandService.preloadOnDemand$.pipe(mergeMap(routePathToPreload => {
            const shouldPreload = this.preloadCheck(route, routePathToPreload);
            if (shouldPreload) {
                return load();
            }
            else {
                return EMPTY;
            }
        }));
    }
    preloadCheck(route, routePathToPreload) {
        var _a;
        if (!route.path) {
            return false;
        }
        const routePathRegex = route.path
            .split('/')
            .map(segment => {
            if (segment.startsWith(':')) {
                return '[a-zA-Z0-9]*';
            }
            else {
                return segment;
            }
        })
            .join('\/');
        const doesMatchPattern = !!((_a = routePathToPreload.match(routePathRegex)) === null || _a === void 0 ? void 0 : _a.length);
        return doesMatchPattern;
    }
}
HoverPreloadStrategy.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: HoverPreloadStrategy, deps: [{ token: PreloadOnDemandService }], target: i0.ɵɵFactoryTarget.Injectable });
HoverPreloadStrategy.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: HoverPreloadStrategy, providedIn: null });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: HoverPreloadStrategy, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: null,
                    deps: [PreloadOnDemandService]
                }]
        }], ctorParameters: function () { return [{ type: PreloadOnDemandService }]; } });

class HoverPreloadModule {
}
HoverPreloadModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: HoverPreloadModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
HoverPreloadModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.2.0", ngImport: i0, type: HoverPreloadModule, declarations: [HoverPreloadLinkDirective], exports: [HoverPreloadLinkDirective] });
HoverPreloadModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: HoverPreloadModule, providers: [
        HoverPreloadStrategy
    ] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.0", ngImport: i0, type: HoverPreloadModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        HoverPreloadLinkDirective,
                    ],
                    imports: [],
                    providers: [
                        HoverPreloadStrategy
                    ],
                    exports: [
                        HoverPreloadLinkDirective,
                    ]
                }]
        }] });

/*
 * Public API Surface of ngx-hover-preload-v2
 */

/**
 * Generated bundle index. Do not edit.
 */

export { HoverPreloadLinkDirective, HoverPreloadModule, HoverPreloadStrategy, PreloadOnDemandService };
//# sourceMappingURL=ngx-hover-preload-v2.mjs.map
